/*
 * MOC - music on console
 * Copyright (C) 2002 Damian Pietras <daper@daper.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#ifndef INTERFACE_H
#define INTERFACE_H

#include <ncurses.h>
#include <pthread.h>
#include "menu.h"
#include "playlist.h"

#define LOCK_SCR() pthread_mutex_lock (&scr_mut)
#define UNLOCK_SCR() pthread_mutex_unlock (&scr_mut)

extern WINDOW *main_win, *main_subwin;
extern Menu *fmenu;
extern pthread_mutex_t scr_mut;
extern short colors;
extern plist *curr_plist;

struct progress_bar {
	int pos[3];
	int direction[3]; /* 1 - right, -1 - left */
	int on;
};
struct progress_bar progress_bar;

void init_interface ();
void events ();
void end_interface ();
void fatal (const char *msg);
void main_title (const char *title);
void set_title (const char *title);
void set_bitrate (const int bitrate);
void set_rate (const int rate);
void set_curr_time (const double time);
void set_tile (const double time);
void set_curr_time (const double time);
void set_time (const double time);
void set_stereo (const short stereo);
void set_state (const char *state);
void show_err (const char *format, ...);
void reset_file_info ();
void mark_item (const char *file);
void unmark_item ();
void set_num_items (int num);
void move_progress_bar ();
void update_info_win ();

#endif
