/*
 * MOC - music on console
 * Copyright (C) 2002 Damian Pietras <daper@daper.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <ncurses.h>
#include "menu.h"

typedef struct {
	char *title;
	char *artist;
	char *album;
} song_info;

typedef struct {
	char *file_name;
	char *title;
	song_info *info;
	MenuItem *menu_item;
	short type;
} plist_item;

typedef struct plist {
	plist_item item;
	struct plist *next;
} plist;

typedef struct {
	Menu *menu;
	WINDOW *menu_win;
	MenuItem **items;
	plist *list;
} plist_menu;

typedef struct shuffled_list {
	plist *item;
	struct shuffled_list *next;
} shuffled_list;

extern plist *playlist, *tmp_list, *hidden_list;
extern shuffled_list *shuffled;
extern pthread_mutex_t hidden_list_mut;

#define LOCK_HLIST()	pthread_mutex_lock(&hidden_list_mut)
#define UNLOCK_HLIST()	pthread_mutex_unlock(&hidden_list_mut)

plist *new_plist ();
void add_to_plist (plist *list, const char *file_name, const char *title,
		song_info *info, const short type, const short sort);
void free_plist_menu (plist_menu *menu);
plist_menu *make_plist_menu (plist *list);
void free_song_info (song_info *info);
plist *dup_item (plist *item);
void add_plist_item (plist *list, plist *item, const short sort);
void free_plist (plist *list);
void delete_plist_item (plist *list, plist *item);
void append_plist (plist *list, plist *new_list);
shuffled_list *gen_shuffled_list (plist *list, plist* first_item);
shuffled_list *find_shuffled (shuffled_list *list, const plist *item);
void free_shuffled_list (shuffled_list *list);
plist *dup_plist (plist *list);

#endif
