/*
 * MOC - music on console
 * Copyright (C) 2002 Damian Pietras <daper@daper.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

/*  Most of this code was taken from:
 *  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas,
 *  Thomas Nilsson and 4Front Technologies
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include "main.h"
#include "interface.h"
#include "audio.h"

#define	WAVE_FORMAT_UNKNOWN		(0x0000)
#define	WAVE_FORMAT_PCM			(0x0001)
#define	WAVE_FORMAT_ADPCM		(0x0002)
#define	WAVE_FORMAT_ALAW		(0x0006)
#define	WAVE_FORMAT_MULAW		(0x0007)
#define	WAVE_FORMAT_OKI_ADPCM		(0x0010)
#define	WAVE_FORMAT_DIGISTD		(0x0015)
#define	WAVE_FORMAT_DIGIFIX		(0x0016)
#define	IBM_FORMAT_MULAW         	(0x0101)
#define	IBM_FORMAT_ALAW			(0x0102)
#define	IBM_FORMAT_ADPCM         	(0x0103)

#define BUFF_SIZE	2048 * 2

typedef struct {
	FILE *file;
	short format_tag, channels, block_align, bits_per_sample, eof, going;
	long samples_per_sec, avg_bytes_per_sec;
	unsigned long position, length;
	int seek_to, data_offset;
	pid_t pid;
} WaveFile;

WaveFile wav_file;
static pthread_mutex_t seek_mut = PTHREAD_MUTEX_INITIALIZER;

static int read_le_long(FILE * file, long *ret) {
	unsigned char buf[4];

	if (fread(buf, 1, 4, file) != 4)
		return 0;

	*ret = (buf[3] << 24) | (buf[2] << 16) | (buf[1] << 8) | buf[0];
	return 1;
}

static int read_le_short(FILE * file, short *ret) {
	unsigned char buf[2];

	if (fread(buf, 1, 2, file) != 2)
		return 0;

	*ret = (buf[1] << 8) | buf[0];
	return 1;
}

int load_wav (char *file_name) {
	char magic[4];
	unsigned long len;
	int rate;

	memset(&wav_file, 0, sizeof (WaveFile));
	if ((wav_file.file = fopen(file_name, "rb")))
	{
		fread(magic, 1, 4, wav_file.file);
		if (strncmp(magic, "RIFF", 4))
		{
			fclose(wav_file.file);
			show_err ("Bad wave header.");
			return 1;
		}
		read_le_long(wav_file.file, &len);
		fread(magic, 1, 4, wav_file.file);
		if (strncmp(magic, "WAVE", 4))
		{
			fclose(wav_file.file);
			show_err ("Bad wave header.");
			return 1;
		}
		for (;;)
		{
			fread(magic, 1, 4, wav_file.file);
			if (!read_le_long(wav_file.file, &len))
			{
				fclose(wav_file.file);
				show_err ("Error in the WAVE file.");
				return 1;
			}
			if (!strncmp("fmt ", magic, 4))
				break;
			fseek(wav_file.file, len, SEEK_CUR);
		}
		if (len < 16)
		{
			fclose(wav_file.file);
			return 1;
		}
		read_le_short(wav_file.file, &wav_file.format_tag);
		switch (wav_file.format_tag)
		{
			case WAVE_FORMAT_UNKNOWN:
			case WAVE_FORMAT_ALAW:
			case WAVE_FORMAT_MULAW:
			case WAVE_FORMAT_ADPCM:
			case WAVE_FORMAT_OKI_ADPCM:
			case WAVE_FORMAT_DIGISTD:
			case WAVE_FORMAT_DIGIFIX:
			case IBM_FORMAT_MULAW:
			case IBM_FORMAT_ALAW:
			case IBM_FORMAT_ADPCM:
				fclose(wav_file.file);
				show_err ("Unknown WAVE format.");
				return 1;
		}
		read_le_short(wav_file.file, &wav_file.channels);
		read_le_long(wav_file.file, &wav_file.samples_per_sec);
		read_le_long(wav_file.file, &wav_file.avg_bytes_per_sec);
		read_le_short(wav_file.file, &wav_file.block_align);
		read_le_short(wav_file.file, &wav_file.bits_per_sample);
		if (wav_file.bits_per_sample != 8 &&
				wav_file.bits_per_sample != 16) {
			fclose(wav_file.file);
			show_err ("Unknown bit per sample value.");
			return 1;
		}
		len -= 16;
		if (len)
			fseek(wav_file.file, len, SEEK_CUR);

		for (;;)
		{
			fread(magic, 4, 1, wav_file.file);

			if (!read_le_long(wav_file.file, &len))
			{
				fclose(wav_file.file);
				show_err ("Error in the WAVE file.");
				return 1;
			}
			if (!strncmp("data", magic, 4))
				break;
			fseek(wav_file.file, len, SEEK_CUR);
		}
		wav_file.data_offset = ftell(wav_file.file);
		wav_file.length = len;

		wav_file.position = 0;
		wav_file.going = 1;

		if (open_audio(16,
					wav_file.channels,
					wav_file.samples_per_sec)) {
			fclose(wav_file.file);
			return 1;
		}
		rate = wav_file.samples_per_sec * wav_file.channels *
			(wav_file.bits_per_sample / 8);
		set_rate (wav_file.samples_per_sec / 1000);
		set_stereo (wav_file.channels == 2 ? 1 : 0);
		set_time (wav_file.length / rate);
		wav_file.seek_to = -1;
	}

	return 0;
}

static void output (char *data, int size) {
	if (wav_file.bits_per_sample == 16)
		play_buff (data, size);
	else {
		char buff[BUFF_SIZE * 2];
		int i, word, pos = 0;

		for (i = 0; i < size; i++) {
			word = (data[i] ^ 128) << 8;
			buff[pos++] = word & 0xFF;
			buff[pos++] = word >> 8;
		}
		

		play_buff (buff, size * 2);
	}
}

void process_wav () {
	char data[BUFF_SIZE];
	int bytes, blk_size, rate;
	int actual_read;
	short stop = 0;
	int seek = -1;

	blk_size = 512 * (wav_file.bits_per_sample / 8) * wav_file.channels;
	rate = wav_file.samples_per_sec * wav_file.channels * (wav_file.bits_per_sample / 8);
	while (wav_file.going)
	{
		if (!wav_file.eof)
		{

			/* Pause? */
			pthread_mutex_lock (&paused_mut);
			if (paused)
				pthread_cond_wait (&paused_cond, &paused_mut);
			pthread_mutex_unlock (&paused_mut);

			/* Stop? */
			pthread_mutex_lock (&req_mut);
			stop = stop_req;
			pthread_mutex_unlock (&req_mut);
			if (stop)
				break;
			
			pthread_mutex_lock (&seek_mut);
			seek = wav_file.seek_to;
			wav_file.seek_to = -1;
			pthread_mutex_unlock (&seek_mut);

			bytes = blk_size;
			if (wav_file.length - wav_file.position < bytes)
				bytes = wav_file.length - wav_file.position;
			if (bytes > 0)
			{
				actual_read = fread(data, 1, bytes, wav_file.file);

				if (actual_read == 0)
				{
					wav_file.eof = 1;
					wav_file.going = 0;
				}
				else
				{
					if(wav_file.going && seek == -1)
						output (data, bytes);
					wav_file.position += actual_read;
					set_curr_time (wav_file.position / rate);
				}
			}
			else
			{
				wav_file.eof = 1;
				wav_file.going = 0;
			}
		}

		if (seek != -1)
		{
			wav_file.position = seek * rate;
			if (wav_file.position < 0)
				wav_file.position = 0;
			fseek(wav_file.file, wav_file.position +
					wav_file.data_offset, SEEK_SET);
		}

	}
	fclose(wav_file.file);
	close_audio ();
}

void seek_wav (int ms) {
	int rate = wav_file.samples_per_sec * wav_file.channels *
			(wav_file.bits_per_sample / 8);
	
	pthread_mutex_lock (&seek_mut);
	wav_file.seek_to = wav_file.position / rate + ms;
	pthread_mutex_unlock (&seek_mut);
}
